/*
 * DemoNe.java
 * www.bouthier.net
 *
 * The MIT License :
 * -----------------
 * Copyright (c) 2003 Christophe Bouthier
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package hypertree.demo;

import hypertree.HyperTree;
import hypertree.HTView;

import java.io.*;
import java.util.Hashtable;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;



/**
 * The DemoNe class implements a demo for HyperTree similar to the Demo class,
 * but testing the idea developed in the Neuchatel seminar.
 * The idea is to change contiuously the projection from Klein to Poincare to
 * the negative infinity.
 * The demo tree is build from a file tree, passed in parameter.
 * Demo could take an argument, the path from which 
 * start the representing of files.
 * If no arguments is given, the treemap start from the root.
 *
 * @author Christophe Bouthier [bouthier@loria.fr]
 * @version 1.0
 */
public class DemoNe {

    private static HTFileNode root      = null; // the root of the demo tree
    private static HyperTree  hypertree = null; // the hypertree builded


    /**
     * Display a demo HyperTree.
     */
    public static void main(String[] args) {
        String pathRoot = null;

        if (args.length > 0) {
            pathRoot = args[0];
        } else {
            pathRoot = ".";
        }

        File rootFile = new File(pathRoot);
        try {
            System.out.println("Starting the hyperbolic tree from " +
                                rootFile.getCanonicalPath());
        } catch (IOException e) {
            e.printStackTrace();
            return;
        }

        if (! rootFile.exists()) {
            System.out.println("Can't start hypertree : " + rootFile.getName() +
                               " does not exist.");
            return;
        }


        root = new HTFileNode(rootFile);
        if (root == null) {
            System.err.println("Error : can't start hypertree from " +
                                rootFile.getAbsolutePath());
            return;
        }

        hypertree = new HyperTree(root);
        HTView view = hypertree.getView();

        // the frame 
        JFrame viewFrame = new JFrame(root.getName());
        JPanel panel = new JPanel(new BorderLayout());
        viewFrame.setContentPane(panel);

        // Hypertree panel
        panel.add(view, BorderLayout.CENTER);

        // Boolean config panel
        JCheckBox boxFastMode = new JCheckBox("Fast mode", false);
        JCheckBox boxLongName = new JCheckBox("Long names", false);
        JCheckBox boxCircle = new JCheckBox("Circle", false);
        JCheckBox boxTrans = new JCheckBox("Translation not corrected", false);
        JCheckBox boxQuad = new JCheckBox("Quad Curves", true);
        boxFastMode.addChangeListener(new BoxFastListener(view, boxFastMode));
        boxLongName.addChangeListener(new BoxNameListener(view, boxLongName));
        boxCircle.addChangeListener(new BoxCircleListener(view, boxCircle));
        boxTrans.addChangeListener(new BoxTransListener(view, boxTrans));
        boxQuad.addChangeListener(new BoxQuadListener(view, boxQuad));
        JPanel panConfBox = new JPanel(new GridLayout(5,1));
        panConfBox.add(boxFastMode);
        panConfBox.add(boxLongName);
        panConfBox.add(boxCircle);
        panConfBox.add(boxTrans);
        panConfBox.add(boxQuad);
        
        // Slider config panel
        JSlider slider = new JSlider(JSlider.HORIZONTAL, -100, 10, 0);
        slider.setMajorTickSpacing(50);
        slider.setMinorTickSpacing(10);
        slider.setPaintLabels(true);
        slider.setPaintTicks(true);
        slider.setPaintTrack(true);
        Hashtable labelTable = new Hashtable();
        labelTable.put(new Integer(10),new JLabel("1"));
        labelTable.put(new Integer(0),new JLabel("0"));
        labelTable.put(new Integer(-10),new JLabel("-1"));
        labelTable.put(new Integer(-50),new JLabel("-5"));
        labelTable.put(new Integer(-100),new JLabel("-10"));
        slider.setLabelTable(labelTable);
        slider.addChangeListener(new SliderListener(view, slider));
        JLabel labSlider = new JLabel("Projection type :");
        JPanel panConfSlider = new JPanel(new GridLayout(2,1));
        panConfSlider.add(labSlider);
        panConfSlider.add(slider);

        // Buttons config panel
        JButton butRestore = new JButton("Restore tree");
        JButton butRestPoinc = new JButton("Restore Poincare model");
        butRestore.addActionListener(new ButRestListener(view));
        butRestPoinc.addActionListener(new ButRestPoincListener(view, slider));
        JPanel panConfBut = new JPanel(new GridLayout(2,1));
        panConfBut.add(butRestore);
        panConfBut.add(butRestPoinc);

        // Layout of the config panel
        JPanel panConf = new JPanel();
        BoxLayout l = new BoxLayout(panConf, BoxLayout.Y_AXIS);
        panConf.setLayout(l);
        panConf.add(panConfBox);
        panConf.add(panConfBut);
        panConf.add(panConfSlider);
        panel.add(panConf, BorderLayout.EAST);

        viewFrame.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        });

        viewFrame.pack();
        viewFrame.setVisible(true);
    }

}


class SliderListener
    implements ChangeListener {

    private HTView v;
    private JSlider s;

    public SliderListener(HTView view, JSlider slider) {
        v = view;
        s = slider;
    }
            
    public void stateChanged(ChangeEvent e) {
        v.setProjType(((double) s.getValue()) / 10);    
        v.repaint();
    }
}

class BoxFastListener 
    implements ChangeListener {

    private HTView v;
    private JCheckBox b;

    public BoxFastListener(HTView view, JCheckBox box) {
        v = view;
        b = box;
    }
            
    public void stateChanged(ChangeEvent e) {
        v.setFastMode(b.isSelected());    
        v.repaint();
    }
}

class BoxNameListener 
    implements ChangeListener {

    private HTView v;
    private JCheckBox b;

    public BoxNameListener(HTView view, JCheckBox box) {
        v = view;
        b = box;
    }
            
    public void stateChanged(ChangeEvent e) {
        v.setLongNameMode(b.isSelected());    
        v.repaint();
    }
}

class BoxCircleListener 
    implements ChangeListener {

    private HTView v;
    private JCheckBox b;

    public BoxCircleListener(HTView view, JCheckBox box) {
        v = view;
        b = box;
    }
            
    public void stateChanged(ChangeEvent e) {
        v.setCircleMode(b.isSelected());    
        v.repaint();
    }
}

class BoxTransListener 
    implements ChangeListener {

    private HTView v;
    private JCheckBox b;

    public BoxTransListener(HTView view, JCheckBox box) {
        v = view;
        b = box;
    }
            
    public void stateChanged(ChangeEvent e) {
        v.setTranslationNotCorrected(b.isSelected());    
        v.repaint();
    }
}

class BoxQuadListener 
    implements ChangeListener {

    private HTView v;
    private JCheckBox b;

    public BoxQuadListener(HTView view, JCheckBox box) {
        v = view;
        b = box;
    }
            
    public void stateChanged(ChangeEvent e) {
        v.setQuadMode(b.isSelected());    
        v.repaint();
    }
}

class ButRestListener 
    implements ActionListener {

    private HTView v;

    public ButRestListener(HTView view) {
        v = view;
    }
            
    public void actionPerformed(ActionEvent e) {
        v.restore();    
        v.repaint();
    }
}

class ButRestPoincListener 
    implements ActionListener {

    private HTView v;
    private JSlider s;

    public ButRestPoincListener(HTView view, JSlider slider) {
        v = view;
        s = slider;
    }
            
    public void actionPerformed(ActionEvent e) {
        v.restorePoincare();    
        s.setValue(0);
        v.repaint();
    }
}

