/*
 * HTNewickNode.java
 * www.bouthier.net
 *
 * The MIT License :
 * -----------------
 * Copyright (c) 2003 Christophe Bouthier
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package hypertree.demo;

import hypertree.HTNode;
import java.util.Enumeration;
import java.util.Vector;
import java.awt.Color;


/**
 * The HTNewickNode implements an example of HTNode encapsulating a Newick 
 * tree format node.
 * <P>
 * The color legend is :
 * <UL>
 *   <IL> blue   for negative length
 *   <IL> white  for length between 0.0 and 1.0
 *   <IL> green  for length between 1.0 and 2.0
 *   <IL> yellow for length between 2.0 and 3.0
 *   <IL> orange for length between 3.0 and 4.0
 *   <IL> red    for length superior to 5.0
 * </UL>
 *
 * @author Christophe Bouthier [bouthier@loria.fr]
 * @version 1.0
 */
public class HTNewickNode
    implements HTNode {

    private String   name     = "";   // node name
    private double   length   = 0.0;  // the branch length, used for color
    private Vector   children = null; // the children of this node
    private boolean  leaf     = true; // is this node a leaf ?


  /* --- Constructor --- */

    /**
     * Constructor, build a leaf.
     */
    public HTNewickNode() {
        this(true);
    }

    /**
     * Constructor. 
     *
     * @param leaf      is this node a leaf ?
     */
    public HTNewickNode(boolean leaf) {
        this.leaf = leaf;
        children = new Vector();
    }


  /* --- Tree management --- */

    /**
     * Add child to the node.
     * 
     * @param child    the HTNewickNode to add as a child
     */
    public void addChild(HTNewickNode child) {
        children.add(child);
    }


  /* --- Accessors --- */

    /**
     * Sets the name of the node.
     *
     * @param name    the name of the node
     */
    public void setName(String name) {
        this.name = name;
    }

    /**
     * Sets the length of the node.
     *
     * @param length    the length of the node
     */
    public void setLength(double length) {
        this.length = length;
    }


  /* --- HTNode --- */

    /**
     * Returns the children of this node in an Enumeration.
     * If this node is a file, return a empty Enumeration.
     * Else, return an Enumeration full with HTFileNode.
     *
     * @return    an Enumeration containing childs of this node
     */
    public Enumeration children() {
        return children.elements();
    }

    /**
     * Always return false in our case.
     *
     * @return    false
     */
    public boolean isLeaf() {
        return leaf;
    }
    
    /**
     * Returns the name of the node.
     *
     * @return    the name of the node
     */
    public String getName() {
        return name;
    }

    /**
     * Returns the color of the node.
     *
     * @return    the color of the node
     */
    public Color getColor() {
        if (length < 0.0) { 
            return Color.blue;
        } else if (length < 1.0) {
            return Color.white;
        } else if (length < 2.0) {
            return Color.green;
        } else if (length < 3.0) {
            return Color.yellow;
        } else if (length < 4.0) {
            return Color.orange;
        } else { 
            return Color.red;
        }
    }


  /* --- ToString --- */

    /**
     * Returns this node and its children
     * as a String.
     *
     * @return    the string form
     */
    public String toString() {
        String s = "node: " + getName();
        if (! isLeaf()) {
            s += "--- children ---";
            for (Enumeration e = children(); e.hasMoreElements(); ) {
                s += "\n" + e.nextElement();
            }
            s += "\n--- end children ---";
        }
        return s;
    }
}

